#! /bin/sh

PATH=/sbin:/bin:/usr/sbin:/usr/bin
DAEMON=/usr/sbin/powernowd
NAME=powernowd
DESC=powernowd

test -x $DAEMON || exit 0

[ -f /etc/default/$NAME ] && . /etc/default/$NAME

set -e

set_governor() {
  	for i in `/bin/ls /sys/devices/system/cpu/`; do
		# FIXME: Get rid of bashisms like pushd/popd
		pushd . >& /dev/null
		cd "/sys/devices/system/cpu/$i/cpufreq"
		if grep -q "$1" < scaling_available_governors; then
			echo "$1" > scaling_governor
		else
			echo "Governor $1 not available for CPU $i. Module loaded?"
		fi
		popd . >& /dev/null
	done
}

print_cpu_status() {
  	for i in `/bin/ls /sys/devices/system/cpu/`; do
		pushd . >& /dev/null
		cd "/sys/devices/system/cpu/$i/cpufreq"
		if [ -f scaling_setspeed ]; then
			echo "$i: $(cat scaling_setspeed)"
		else
			echo "$i: $(cat scaling_governor)"
		fi
		popd . >& /dev/null
	done
}

case "$1" in
  start)
	echo -n "Starting $DESC: "
	if [ -f /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor ]
	then
		start-stop-daemon --start --quiet --oknodo --exec $DAEMON -- $OPTIONS
	else
		echo "required sysfs objects not found!"
		echo -e "\tRead /usr/share/doc/powernowd/README.Debian for more information."
		exit 0
	fi
	echo "$NAME."
	;;
  stop)
	echo -n "Stopping $DESC: "
	start-stop-daemon --stop --quiet --oknodo --exec $DAEMON
	echo "$NAME."
	print_cpu_status
	;;
  restart|force-reload)
	echo -n "Restarting $DESC: "
	start-stop-daemon --stop --quiet --oknodo --exec $DAEMON
	sleep 1
	start-stop-daemon --start --quiet --exec $DAEMON -- $OPTIONS
	echo "$NAME."
	;;
  powersave|performance)
    	$0 stop
	set_governor "$1"
	;;
  low|high)
    	$0 stop
	set_governor "userspace"
	for i in `/bin/ls /sys/devices/system/cpu/`; do 
		pushd . >& /dev/null
		cd "/sys/devices/system/cpu/$i/cpufreq"
		if [ "$1" = "low" ]; then
			cat scaling_min_freq > scaling_setspeed
		else
			cat scaling_max_freq > scaling_setspeed
		fi
		popd >& /dev/null
	done
	print_cpu_status
	;;
  status)
  	echo -n "powernowd state: "
	if ps ax | grep -q "[ ]$DAEMON"; then
	    echo "running"
	else
	    echo "stopped"
	fi
	print_cpu_status
	;;
  *)
	N=/etc/init.d/$NAME
	echo "Usage: $N {start|stop|restart|force-reload|high|low}" >&2
	exit 1
	;;
esac

exit 0
